package Dancer2::CLI;
# ABSTRACT: Dancer2 cli application
$Dancer2::CLI::VERSION = '0.200001';
use strict;
use warnings;
use App::Cmd::Setup -app;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Dancer2::CLI - Dancer2 cli application

=head1 VERSION

version 0.200001

=head1 AUTHOR

Dancer Core Developers

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2016 by Alexis Sukrieh.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
